use cgmath::{Deg, InnerSpace, Point3, Vector2};
use galactica_content::{FactionHandle, TextureHandle};
use rapier2d::{
	dynamics::{RigidBody, RigidBodyBuilder, RigidBodyHandle},
	geometry::{ColliderBuilder, ColliderHandle},
};

use crate::{physics::util, render::ObjectSprite};

pub struct ProjectileBuilder {
	pub rigid_body: RigidBodyBuilder,
	pub collider: ColliderBuilder,
	pub sprite_texture: TextureHandle,
	pub lifetime: f32,
	pub size: f32,
	pub damage: f32,
	pub faction: FactionHandle,
}

impl ProjectileBuilder {
	pub fn build(self, r: RigidBodyHandle, c: ColliderHandle) -> Projectile {
		Projectile {
			rigid_body: r,
			collider: c,
			sprite_texture: self.sprite_texture,
			lifetime: self.lifetime,
			size: self.size,
			damage: self.damage,
			faction: self.faction,
		}
	}
}

#[derive(Debug)]
pub struct Projectile {
	pub rigid_body: RigidBodyHandle,
	pub collider: ColliderHandle,
	pub sprite_texture: TextureHandle,
	pub lifetime: f32,
	pub size: f32,
	pub damage: f32,
	pub faction: FactionHandle,
}

impl Projectile {
	pub fn tick(&mut self, t: f32) {
		self.lifetime -= t;
	}

	pub fn is_expired(&self) -> bool {
		return self.lifetime < 0.0;
	}

	pub fn get_sprite(&self, r: &RigidBody) -> ObjectSprite {
		let pos = util::rigidbody_position(r);
		let rot = util::rigidbody_rotation(r);

		// Sprites point north at 0 degrees
		let ang: Deg<f32> = rot.angle(Vector2 { x: 1.0, y: 0.0 }).into();

		ObjectSprite {
			texture: self.sprite_texture,
			pos: Point3 {
				x: pos.x,
				y: pos.y,
				z: 1.0,
			},
			size: self.size,
			angle: -ang,
			children: None,
		}
	}
}
